@@ -1,10 +1,9 @@
 //! A graph representation for computing tree diffs.
 
 use bumpalo::Bump;
-use rpds::Stack;
 use rustc_hash::FxHashMap;
 use std::{
-    cell::RefCell,
+    cell::{Cell, RefCell},
     cmp::min,
     fmt,
     hash::{Hash, Hasher},
@@ -12,7 +11,10 @@ use std::{
 use strsim::normalized_levenshtein;
 
 use crate::{
-    diff::changes::{insert_deep_unchanged, ChangeKind, ChangeMap},
+    diff::{
+        changes::{insert_deep_unchanged, ChangeKind, ChangeMap},
+        stack::Stack,
+    },
     parse::syntax::{AtomKind, Syntax, SyntaxId},
 };
 use Edge::*;
@@ -48,43 +50,50 @@ use Edge::*;
 #[derive(Debug, Clone)]
 pub struct Vertex<'a, 'b> {
     pub neighbours: RefCell<Option<Vec<(Edge, &'b Vertex<'a, 'b>)>>>,
-    pub predecessor: RefCell<Option<(u64, &'b Vertex<'a, 'b>)>>,
+    pub predecessor: Cell<Option<(u64, &'b Vertex<'a, 'b>)>>,
     pub lhs_syntax: Option<&'a Syntax<'a>>,
     pub rhs_syntax: Option<&'a Syntax<'a>>,
     parents: Stack<EnteredDelimiter<'a>>,
     lhs_parent_id: Option<SyntaxId>,
     rhs_parent_id: Option<SyntaxId>,
-    can_pop_either: bool,
 }
 
 impl<'a, 'b> PartialEq for Vertex<'a, 'b> {
     fn eq(&self, other: &Self) -> bool {
-        self.lhs_syntax.map(|node| node.id()) == other.lhs_syntax.map(|node| node.id())
-            && self.rhs_syntax.map(|node| node.id()) == other.rhs_syntax.map(|node| node.id())
-            // Strictly speaking, we should compare the whole
-            // EnteredDelimiter stack, not just the immediate
-            // parents. By taking the immediate parent, we have
-            // vertices with different stacks that are 'equal'.
-            //
-            // This makes the graph traversal path dependent: the
-            // first vertex we see 'wins', and we use it for deciding
-            // how we can pop.
-            //
-            // In practice this seems to work well. The first vertex
-            // has the lowest cost, so has the most PopBoth
-            // occurrences, which is the best outcome.
-            //
-            // Handling this properly would require considering many
-            // more vertices to be distinct, exponentially increasing
-            // the graph size relative to tree depth.
-            && self.lhs_parent_id == other.lhs_parent_id
-            && self.rhs_parent_id == other.rhs_parent_id
-            // We do want to distinguish whether we can pop each side
-            // independently though. Without this, if we find a case
-            // where we can pop sides together, we don't consider the
-            // case where we get a better diff by popping each side
-            // separately.
-            && self.can_pop_either == other.can_pop_either
+        // Strictly speaking, we should compare the whole
+        // EnteredDelimiter stack, not just the immediate
+        // parents. By taking the immediate parent, we have
+        // vertices with different stacks that are 'equal'.
+        //
+        // This makes the graph traversal path dependent: the
+        // first vertex we see 'wins', and we use it for deciding
+        // how we can pop.
+        //
+        // In practice this seems to work well. The first vertex
+        // has the lowest cost, so has the most PopBoth
+        // occurrences, which is the best outcome.
+        //
+        // Handling this properly would require considering many
+        // more vertices to be distinct, exponentially increasing
+        // the graph size relative to tree depth.
+        let b0 = match (self.lhs_syntax, other.lhs_syntax) {
+            (Some(s0), Some(s1)) => s0.id() == s1.id(),
+            (None, None) => self.lhs_parent_id == other.lhs_parent_id,
+            _ => false,
+        };
+        let b1 = match (self.rhs_syntax, other.rhs_syntax) {
+            (Some(s0), Some(s1)) => s0.id() == s1.id(),
+            (None, None) => self.rhs_parent_id == other.rhs_parent_id,
+            _ => false,
+        };
+        // We do want to distinguish whether we can pop each side
+        // independently though. Without this, if we find a case
+        // where we can pop sides together, we don't consider the
+        // case where we get a better diff by popping each side
+        // separately.
+        let b2 = can_pop_either_parent(&self.parents) == can_pop_either_parent(&other.parents);
+
+        b0 && b1 && b2
     }
 }
 impl<'a, 'b> Eq for Vertex<'a, 'b> {}
@@ -96,7 +105,7 @@ impl<'a, 'b> Hash for Vertex<'a, 'b> {
 
         self.lhs_parent_id.hash(state);
         self.rhs_parent_id.hash(state);
-        self.can_pop_either.hash(state);
+        can_pop_either_parent(&self.parents).hash(state);
     }
 }
 
@@ -209,44 +218,30 @@ fn push_lhs_delimiter<'a>(
     entered: &Stack<EnteredDelimiter<'a>>,
     delimiter: &'a Syntax<'a>,
 ) -> Stack<EnteredDelimiter<'a>> {
-    let mut modifying_head = false;
-    let (mut lhs_delims, rhs_delims) = match entered.peek() {
-        Some(EnteredDelimiter::PopEither((lhs_delims, rhs_delims))) => {
-            modifying_head = true;
-            (lhs_delims.clone(), rhs_delims.clone())
-        }
-        _ => (Stack::new(), Stack::new()),
-    };
-    lhs_delims = lhs_delims.push(delimiter);
-
-    let entered = if modifying_head {
-        entered.pop().unwrap()
-    } else {
-        entered.clone()
-    };
-    entered.push(EnteredDelimiter::PopEither((lhs_delims, rhs_delims)))
+    match entered.peek() {
+        Some(EnteredDelimiter::PopEither((lhs_delims, rhs_delims))) => entered.pop().unwrap().push(
+            EnteredDelimiter::PopEither((lhs_delims.push(delimiter), rhs_delims.clone())),
+        ),
+        _ => entered.push(EnteredDelimiter::PopEither((
+            Stack::new().push(delimiter),
+            Stack::new(),
+        ))),
+    }
 }
 
 fn push_rhs_delimiter<'a>(
     entered: &Stack<EnteredDelimiter<'a>>,
     delimiter: &'a Syntax<'a>,
 ) -> Stack<EnteredDelimiter<'a>> {
-    let mut modifying_head = false;
-    let (lhs_delims, mut rhs_delims) = match entered.peek() {
-        Some(EnteredDelimiter::PopEither((lhs_delims, rhs_delims))) => {
-            modifying_head = true;
-            (lhs_delims.clone(), rhs_delims.clone())
-        }
-        _ => (Stack::new(), Stack::new()),
-    };
-    rhs_delims = rhs_delims.push(delimiter);
-
-    let entered = if modifying_head {
-        entered.pop().unwrap()
-    } else {
-        entered.clone()
-    };
-    entered.push(EnteredDelimiter::PopEither((lhs_delims, rhs_delims)))
+    match entered.peek() {
+        Some(EnteredDelimiter::PopEither((lhs_delims, rhs_delims))) => entered.pop().unwrap().push(
+            EnteredDelimiter::PopEither((lhs_delims.clone(), rhs_delims.push(delimiter))),
+        ),
+        _ => entered.push(EnteredDelimiter::PopEither((
+            Stack::new(),
+            Stack::new().push(delimiter),
+        ))),
+    }
 }
 
 impl<'a, 'b> Vertex<'a, 'b> {
@@ -258,13 +253,12 @@ impl<'a, 'b> Vertex<'a, 'b> {
         let parents = Stack::new();
         Vertex {
             neighbours: RefCell::new(None),
-            predecessor: RefCell::new(None),
+            predecessor: Cell::new(None),
             lhs_syntax,
             rhs_syntax,
             parents,
             lhs_parent_id: None,
             rhs_parent_id: None,
-            can_pop_either: false,
         }
     }
 }
@@ -414,7 +408,7 @@ fn allocate_if_new<'syn, 'b>(
 /// This check is deliberately conservative, becuase it's hard to
 /// accurately recognise punctuation in a language-agnostic way.
 fn looks_like_punctuation(content: &str) -> bool {
-    content == "," || content == ";"
+    content == "," || content == ";" || content == "."
 }
 
 /// Compute the neighbours of `v` if we haven't previously done so,
@@ -442,10 +436,9 @@ pub fn get_set_neighbours<'syn, 'b>(
                 allocate_if_new(
                     Vertex {
                         neighbours: RefCell::new(None),
-                        predecessor: RefCell::new(None),
+                        predecessor: Cell::new(None),
                         lhs_syntax: lhs_parent.next_sibling(),
                         rhs_syntax: rhs_parent.next_sibling(),
-                        can_pop_either: can_pop_either_parent(&parents_next),
                         parents: parents_next,
                         lhs_parent_id: lhs_parent.parent().map(Syntax::id),
                         rhs_parent_id: rhs_parent.parent().map(Syntax::id),
@@ -467,10 +460,9 @@ pub fn get_set_neighbours<'syn, 'b>(
                 allocate_if_new(
                     Vertex {
                         neighbours: RefCell::new(None),
-                        predecessor: RefCell::new(None),
+                        predecessor: Cell::new(None),
                         lhs_syntax: lhs_parent.next_sibling(),
                         rhs_syntax: v.rhs_syntax,
-                        can_pop_either: can_pop_either_parent(&parents_next),
                         parents: parents_next,
                         lhs_parent_id: lhs_parent.parent().map(Syntax::id),
                         rhs_parent_id: v.rhs_parent_id,
@@ -492,10 +484,9 @@ pub fn get_set_neighbours<'syn, 'b>(
                 allocate_if_new(
                     Vertex {
                         neighbours: RefCell::new(None),
-                        predecessor: RefCell::new(None),
+                        predecessor: Cell::new(None),
                         lhs_syntax: v.lhs_syntax,
                         rhs_syntax: rhs_parent.next_sibling(),
-                        can_pop_either: can_pop_either_parent(&parents_next),
                         parents: parents_next,
                         lhs_parent_id: v.lhs_parent_id,
                         rhs_parent_id: rhs_parent.parent().map(Syntax::id),
@@ -519,13 +510,12 @@ pub fn get_set_neighbours<'syn, 'b>(
                 allocate_if_new(
                     Vertex {
                         neighbours: RefCell::new(None),
-                        predecessor: RefCell::new(None),
+                        predecessor: Cell::new(None),
                         lhs_syntax: lhs_syntax.next_sibling(),
                         rhs_syntax: rhs_syntax.next_sibling(),
                         parents: v.parents.clone(),
                         lhs_parent_id: v.lhs_parent_id,
                         rhs_parent_id: v.rhs_parent_id,
-                        can_pop_either: v.can_pop_either,
                     },
                     alloc,
                     seen,
@@ -565,13 +555,12 @@ pub fn get_set_neighbours<'syn, 'b>(
                     allocate_if_new(
                         Vertex {
                             neighbours: RefCell::new(None),
-                            predecessor: RefCell::new(None),
+                            predecessor: Cell::new(None),
                             lhs_syntax: lhs_next,
                             rhs_syntax: rhs_next,
                             parents: parents_next,
                             lhs_parent_id: Some(lhs_syntax.id()),
                             rhs_parent_id: Some(rhs_syntax.id()),
-                            can_pop_either: false,
                         },
                         alloc,
                         seen,
@@ -603,13 +592,12 @@ pub fn get_set_neighbours<'syn, 'b>(
                     allocate_if_new(
                         Vertex {
                             neighbours: RefCell::new(None),
-                            predecessor: RefCell::new(None),
+                            predecessor: Cell::new(None),
                             lhs_syntax: lhs_syntax.next_sibling(),
                             rhs_syntax: rhs_syntax.next_sibling(),
                             parents: v.parents.clone(),
                             lhs_parent_id: v.lhs_parent_id,
                             rhs_parent_id: v.rhs_parent_id,
-                            can_pop_either: v.can_pop_either,
                         },
                         alloc,
                         seen,
@@ -633,13 +621,12 @@ pub fn get_set_neighbours<'syn, 'b>(
                     allocate_if_new(
                         Vertex {
                             neighbours: RefCell::new(None),
-                            predecessor: RefCell::new(None),
+                            predecessor: Cell::new(None),
                             lhs_syntax: lhs_syntax.next_sibling(),
                             rhs_syntax: v.rhs_syntax,
                             parents: v.parents.clone(),
                             lhs_parent_id: v.lhs_parent_id,
                             rhs_parent_id: v.rhs_parent_id,
-                            can_pop_either: v.can_pop_either,
                         },
                         alloc,
                         seen,
@@ -659,13 +646,12 @@ pub fn get_set_neighbours<'syn, 'b>(
                     allocate_if_new(
                         Vertex {
                             neighbours: RefCell::new(None),
-                            predecessor: RefCell::new(None),
+                            predecessor: Cell::new(None),
                             lhs_syntax: lhs_next,
                             rhs_syntax: v.rhs_syntax,
                             parents: parents_next,
                             lhs_parent_id: Some(lhs_syntax.id()),
                             rhs_parent_id: v.rhs_parent_id,
-                            can_pop_either: true,
                         },
                         alloc,
                         seen,
@@ -687,13 +673,12 @@ pub fn get_set_neighbours<'syn, 'b>(
                     allocate_if_new(
                         Vertex {
                             neighbours: RefCell::new(None),
-                            predecessor: RefCell::new(None),
+                            predecessor: Cell::new(None),
                             lhs_syntax: v.lhs_syntax,
                             rhs_syntax: rhs_syntax.next_sibling(),
                             parents: v.parents.clone(),
                             lhs_parent_id: v.lhs_parent_id,
                             rhs_parent_id: v.rhs_parent_id,
-                            can_pop_either: v.can_pop_either,
                         },
                         alloc,
                         seen,
@@ -713,13 +698,12 @@ pub fn get_set_neighbours<'syn, 'b>(
                     allocate_if_new(
                         Vertex {
                             neighbours: RefCell::new(None),
-                            predecessor: RefCell::new(None),
+                            predecessor: Cell::new(None),
                             lhs_syntax: v.lhs_syntax,
                             rhs_syntax: rhs_next,
                             parents: parents_next,
                             lhs_parent_id: v.lhs_parent_id,
                             rhs_parent_id: Some(rhs_syntax.id()),
-                            can_pop_either: true,
                         },
                         alloc,
                         seen,
@@ -729,7 +713,7 @@ pub fn get_set_neighbours<'syn, 'b>(
         }
     }
     assert!(
-        res.len() > 0,
+        !res.is_empty(),
         "Must always find some next steps if node is not the end"
     );
 
