@@ -73,7 +73,7 @@ pub fn print(
         let hunk_lines = hunk.lines.clone();
 
         let before_lines =
-            calculate_before_context(&hunk_lines, &opposite_to_lhs, &opposite_to_rhs);
+            calculate_before_context(&hunk_lines, &opposite_to_lhs, &opposite_to_rhs, 3);
         let after_lines = calculate_after_context(
             &[&before_lines[..], &hunk_lines[..]].concat(),
             &opposite_to_lhs,
@@ -81,6 +81,7 @@ pub fn print(
             // TODO: repeatedly calculating the maximum is wasteful.
             lhs_src.max_line(),
             rhs_src.max_line(),
+            display_options.num_context_lines as usize,
         );
 
         for (lhs_line, _) in before_lines {
