@@ -14,7 +14,7 @@
 #![allow(clippy::too_many_arguments)]
 // Has false positives on else if chains that sometimes have the same
 // body for readability.
-#![allow(clippy::clippy::if_same_then_else)]
+#![allow(clippy::if_same_then_else)]
 
 mod constants;
 mod diff;
@@ -52,7 +52,7 @@ use parse::guess_language::{guess, language_name};
 static GLOBAL: MiMalloc = MiMalloc;
 
 use diff::sliders::fix_all_sliders;
-use options::{DisplayMode, DisplayOptions, Mode, DEFAULT_TAB_WIDTH};
+use options::{DisplayMode, DisplayOptions, FileArgument, Mode, DEFAULT_TAB_WIDTH};
 use owo_colors::OwoColorize;
 use rayon::prelude::*;
 use std::{env, path::Path};
@@ -132,12 +132,16 @@ fn main() {
                 }
             }
         }
-        Mode::ListLanguages => {
+        Mode::ListLanguages { use_color } => {
             for (language, extensions) in LANG_EXTENSIONS {
-                print!("{}", language_name(*language).bold());
+                let mut name = language_name(*language).to_string();
+                if use_color {
+                    name = name.bold().to_string();
+                }
+                print!("{}", name);
 
                 let mut extensions: Vec<&str> = (*extensions).into();
-                extensions.sort();
+                extensions.sort_unstable();
 
                 for extension in extensions {
                     print!(" .{}", extension);
@@ -156,45 +160,49 @@ fn main() {
             lhs_display_path,
             rhs_display_path,
         } => {
-            let lhs_path = Path::new(&lhs_path);
-            let rhs_path = Path::new(&rhs_path);
-
             if lhs_path == rhs_path {
+                let is_dir = match &lhs_path {
+                    FileArgument::NamedPath(path) => path.is_dir(),
+                    _ => false,
+                };
+
                 eprintln!(
                     "warning: You've specified the same {} twice.\n",
-                    if lhs_path.is_dir() {
-                        "directory"
-                    } else {
-                        "file"
-                    }
+                    if is_dir { "directory" } else { "file" }
                 );
             }
 
-            if lhs_path.is_dir() && rhs_path.is_dir() {
-                diff_directories(
-                    lhs_path,
-                    rhs_path,
-                    &display_options,
-                    graph_limit,
-                    byte_limit,
-                    language_override,
-                )
-                .for_each(|diff_result| {
+            match (&lhs_path, &rhs_path) {
+                (
+                    options::FileArgument::NamedPath(lhs_path),
+                    options::FileArgument::NamedPath(rhs_path),
+                ) if lhs_path.is_dir() && rhs_path.is_dir() => {
+                    diff_directories(
+                        lhs_path,
+                        rhs_path,
+                        &display_options,
+                        graph_limit,
+                        byte_limit,
+                        language_override,
+                    )
+                    .for_each(|diff_result| {
+                        print_diff_result(&display_options, &diff_result);
+                    });
+                }
+                _ => {
+                    let diff_result = diff_file(
+                        &lhs_display_path,
+                        &rhs_display_path,
+                        &lhs_path,
+                        &rhs_path,
+                        &display_options,
+                        missing_as_empty,
+                        graph_limit,
+                        byte_limit,
+                        language_override,
+                    );
                     print_diff_result(&display_options, &diff_result);
-                });
-            } else {
-                let diff_result = diff_file(
-                    &lhs_display_path,
-                    &rhs_display_path,
-                    lhs_path,
-                    rhs_path,
-                    &display_options,
-                    missing_as_empty,
-                    graph_limit,
-                    byte_limit,
-                    language_override,
-                );
-                print_diff_result(&display_options, &diff_result);
+                }
             }
         }
     };
@@ -214,8 +222,8 @@ fn replace_tabs(src: &str, tab_width: usize) -> String {
 fn diff_file(
     lhs_display_path: &str,
     rhs_display_path: &str,
-    lhs_path: &Path,
-    rhs_path: &Path,
+    lhs_path: &FileArgument,
+    rhs_path: &FileArgument,
     display_options: &DisplayOptions,
     missing_as_empty: bool,
     graph_limit: usize,
@@ -226,6 +234,8 @@ fn diff_file(
     diff_file_content(
         lhs_display_path,
         rhs_display_path,
+        lhs_path,
+        rhs_path,
         &lhs_bytes,
         &rhs_bytes,
         display_options.tab_width,
@@ -238,6 +248,8 @@ fn diff_file(
 fn diff_file_content(
     lhs_display_path: &str,
     rhs_display_path: &str,
+    _lhs_path: &FileArgument,
+    rhs_path: &FileArgument,
     lhs_bytes: &[u8],
     rhs_bytes: &[u8],
     tab_width: usize,
@@ -275,13 +287,12 @@ fn diff_file_content(
         rhs_src.pop();
     }
 
-    // Prefer the RHS path for language detection, unless it's /dev/null.
-    let (guess_src, guess_path) = if rhs_display_path == "/dev/null" {
-        // TODO: take a Path directly instead.
-        (&lhs_src, Path::new(&lhs_display_path))
-    } else {
-        (&rhs_src, Path::new(&rhs_display_path))
+    let (guess_src, guess_path) = match rhs_path {
+        FileArgument::NamedPath(_) => (&rhs_src, Path::new(&rhs_display_path)),
+        FileArgument::Stdin => (&rhs_src, Path::new(&lhs_display_path)),
+        FileArgument::DevNull => (&lhs_src, Path::new(&lhs_display_path)),
     };
+
     let language = language_override.or_else(|| guess(guess_path, guess_src));
     let lang_config = language.map(tsp::from_language);
 
@@ -416,9 +427,9 @@ fn diff_directories<'a>(
 
         diff_file(
             &rel_path.to_string_lossy(),
-            &rel_path.to_string_lossy(), // todo
-            &lhs_path,
-            &rhs_path,
+            &rel_path.to_string_lossy(),
+            &FileArgument::NamedPath(lhs_path),
+            &FileArgument::NamedPath(rhs_path),
             &display_options,
             true,
             graph_limit,
@@ -441,6 +452,7 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
                 &opposite_to_rhs,
                 lhs_src.max_line(),
                 rhs_src.max_line(),
+                display_options.num_context_lines as usize,
             );
 
             let lang_name = summary.language.clone().unwrap_or_else(|| "Text".into());
@@ -540,6 +552,8 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
 
 #[cfg(test)]
 mod tests {
+    use std::ffi::OsStr;
+
     use super::*;
     use crate::options::{DEFAULT_BYTE_LIMIT, DEFAULT_GRAPH_LIMIT, DEFAULT_TAB_WIDTH};
 
@@ -549,6 +563,8 @@ mod tests {
         let res = diff_file_content(
             "foo.el",
             "foo.el",
+            &FileArgument::from_path_argument(OsStr::new("foo.el")),
+            &FileArgument::from_path_argument(OsStr::new("foo.el")),
             s.as_bytes(),
             s.as_bytes(),
             DEFAULT_TAB_WIDTH,
